@@ -16,22 +16,35 @@ class TestSmartSQL(unittest.TestCase):
 
     maxDiff = None
 
-    def test_nested_join(self):
+    def test_join(self):
         t1, t2, t3, t4 = T.t1, T.t2, T.t3, T.t4
+        self.assertEqual(
+            QS((t1 + t2.on(t2.t1_id == t1.id)) * t3.on(t3.t2_id == t2.id) - t4.on(t4.t3_id == t3.id)).select(t1.id),
+            ('SELECT "t1"."id" FROM "t1" LEFT OUTER JOIN "t2" ON ("t2"."t1_id" = "t1"."id") CROSS JOIN "t3" ON ("t3"."t2_id" = "t2"."id") RIGHT OUTER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")', [], )
+        )
+        self.assertEqual(
+            QS((t1 + t2).on(t2.t1_id == t1.id) * t3.on(t3.t2_id == t2.id) - t4.on(t4.t3_id == t3.id)).select(t1.id),
+            ('SELECT "t1"."id" FROM "t1" LEFT OUTER JOIN "t2" ON ("t2"."t1_id" = "t1"."id") CROSS JOIN "t3" ON ("t3"."t2_id" = "t2"."id") RIGHT OUTER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")', [], )
+        )
+        self.assertEqual(
+            QS((t1 + (t2 * t3).on(t3.t2_id == t2.id)).on(t2.t1_id == t1.id) - t4.on(t4.t3_id == t3.id)).select(t1.id),
+            ('SELECT "t1"."id" FROM "t1" LEFT OUTER JOIN ("t2" CROSS JOIN "t3" ON ("t3"."t2_id" = "t2"."id")) ON ("t2"."t1_id" = "t1"."id") RIGHT OUTER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")', [], )
+        )
+        self.assertEqual(
+            QS(((t1 + t2) * t3 - t4).group().on((t2.t1_id == t1.id) & (t3.t2_id == t2.id) & (t4.t3_id == t3.id))).select(t1.id),
+            ('SELECT "t1"."id" FROM ("t1" LEFT OUTER JOIN "t2" CROSS JOIN "t3" RIGHT OUTER JOIN "t4") ON ((("t2"."t1_id" = "t1"."id") AND ("t3"."t2_id" = "t2"."id")) AND ("t4"."t3_id" = "t3"."id"))', [], )
+        )
         self.assertEqual(
             QS((t1 & t2.on(t2.t1_id == t1.id) & (t3 & t4.on(t4.t3_id == t3.id))).on(t3.t2_id == t2.id)).select(t1.id),
-            ('SELECT "t1"."id" FROM "t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id") INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")',
- [], )
+            ('SELECT "t1"."id" FROM "t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id") INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")', [], )
         )
         self.assertEqual(
             QS(t1 & t2.on(t2.t1_id == t1.id) & (t3 & t4.on(t4.t3_id == t3.id)).as_nested().on(t3.t2_id == t2.id)).select(t1.id),
-            ('SELECT "t1"."id" FROM "t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id") INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")',
- [], )
+            ('SELECT "t1"."id" FROM "t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id") INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")', [], )
         )
         self.assertEqual(
             QS((t1 & t2.on(t2.t1_id == t1.id)).group() & (t3 & t4.on(t4.t3_id == t3.id)).group().on(t3.t2_id == t2.id)).select(t1.id),
-            ('SELECT "t1"."id" FROM ("t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id")) INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")',
- [], )
+            ('SELECT "t1"."id" FROM ("t1" INNER JOIN "t2" ON ("t2"."t1_id" = "t1"."id")) INNER JOIN ("t3" INNER JOIN "t4" ON ("t4"."t3_id" = "t3"."id")) ON ("t3"."t2_id" = "t2"."id")', [], )
         )
 
     def test_index(self):
@@ -60,8 +73,7 @@ class TestSmartSQL(unittest.TestCase):
         q = QS(t1 & t2.use_index('index1', 'index2').on(t2.parent_id == t1.id)).dialect('mysql')
         self.assertEqual(
             q.select(t2.id),
-            ('SELECT `al2`.`id` FROM `tb1` INNER JOIN `tb1` AS `al2` USE INDEX (`index1`, `index2`) ON (`al2`.`parent_id` = `tb1`.`id`)',
- [], )
+            ('SELECT `al2`.`id` FROM `tb1` INNER JOIN `tb1` AS `al2` USE INDEX (`index1`, `index2`) ON (`al2`.`parent_id` = `tb1`.`id`)', [], )
         )
 
     def test_prefix(self):
